# Des tableaux avec ng-material: md-table
![category](développement)
![tag](design)
![tag](angularjs)

Il existe dans le monde javascript le module ng-table qui permet de gérer des tableaux.

Le principal problème de ce module est l'intégration graphique avec ngmaterial: 
cela n'est pas génial.

Il existe un module spécifique pour ngmatérial: *md-table*

Ce qui est attendu de la mise en place d'un tableau:

- gestion des colonnes
- trie / filtre dynamique
- pagination
- export des données

Environnement technique
-----------------------

Afin de mettre en évidence le module **ng-table** nous allons utiliser un environnement
docker

![201607011347_1.png](./upload/201607011347_1.png)

Notre fichier docker-compose.yml

```
version: '2'
services:
    front:
        build: front/.
        ports:
            - "80:80"
        volumes:
            - /root/test/data:/app
```

Le fichier dockerfile permettant l'intégration des modules angular

```
FROM nginx:1.9

COPY ./src/default.conf /etc/nginx/conf.d/default.conf

RUN apt-get update && apt-get install -y \
    git \
    git-core \
    nodejs \
    npm
RUN ln -s /usr/bin/nodejs /usr/bin/node
RUN echo '{ "allow_root": true }' > /root/.bowerrc

#################### application ##############################################
RUN npm install -g bower
RUN cd /usr/share/nginx/html && bower install angular-material
RUN cd /usr/share/nginx/html && bower install angular-fontawesome
RUN cd /usr/share/nginx/html && bower install angular-material-data-table
RUN cd /usr/share/nginx/html && bower install ng-csv

RUN mkdir /usr/share/nginx/html/app
RUN ln -s /usr/share/nginx/html/app /app
RUN chmod -R 777 /app
VOLUME /app
```

La configuration du serveur nginx utilisé

```
server {
    listen       80;
    server_name  localhost;
    location / {
        root   /usr/share/nginx/html/app;
        index  index.html index.htm;
    }
    location /bower_components {
        alias   /usr/share/nginx/html/bower_components;
    }
}
```

Pour lancer notre container

```bash
docker-compose up -d 
```

L'ensemble des fichiers html on la structure suivante

```html
<!DOCTYPE html>
<html lang="fr">
    <head>
        <title>Test</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, user-scalable=no, initial-scale=1.0, maximum-scale=1.0, minimal-ui" />
        <!-- Angular -->
        <script src="./bower_components/angular/angular.min.js"></script>
        <!-- Angular Material -->
        <script src="./bower_components/angular-animate/angular-animate.min.js"></script>
        <script src="./bower_components/angular-aria/angular-aria.min.js"></script>
        <script src="./bower_components/angular-material/angular-material.min.js"></script>
        <link rel="stylesheet" href="./bower_components/angular-material/angular-material.min.css" />
        <!-- Angular FontAwesome -->
        <link rel="stylesheet" href="./bower_components/font-awesome/css/font-awesome.min.css">
        <script src="./bower_components/angular-fontawesome/dist/angular-fontawesome.min.js"></script>
        <!-- md-table --><!-- style sheet -->
        <link href="bower_components/angular-material-data-table/dist/md-data-table.min.css" rel="stylesheet" type="text/css"/>
        <!-- module -->
        <script type="text/javascript" src="bower_components/angular-material-data-table/dist/md-data-table.min.js"></script>
        
    </head>
    <body ng-app="MyAppli">
        <div ng-controller="AppController" layout="column" layout-fill>
            <div ui-view="content" layout-fill layout="column" layout-align="center none">
                <!-- table -->
                <!-- fin table -->                       
                        
            </div>
        </div>
        <script>
            <!-- javascript -->
            var app = angular.module('MyAppli', ['picardy.fontawesome','ngMaterial', 'md.data.table']);
            app.controller('AppController', function($scope, $http) {
            
                $scope.getInit = function() {
                    $http
                        .get("./conf/data.json")
                        .success(function(data, status, headers, config) {
                            $scope.datas =data;                            
                        })
                        .error(function(data, status, headers, config) {
                            alert('error');
                        });
                };     
                // init data
                $scope.getInit();
            });
            <!-- fin javascript -->
        </script>
    </body>
</html>
```

notre fichier conf de données est le suivante

```json
[
    {"id":1, "nom":"nom1", "prenom":"prenom1"},
    {"id":2, "nom":"nom2", "prenom":"prenom2"},
    {"id":3, "nom":"nom3", "prenom":"prenom3"},
    {"id":4, "nom":"nom4", "prenom":"prenom4"},
    {"id":5, "nom":"nom5", "prenom":"prenom5"},
    {"id":6, "nom":"nom6", "prenom":"prenom6"},
    {"id":7, "nom":"nom7", "prenom":"prenom7"},
    {"id":8, "nom":"nom8", "prenom":"prenom8"},
    {"id":9, "nom":"nom9", "prenom":"prenom9"},
    {"id":10, "nom":"nom10", "prenom":"prenom1"},
    {"id":11, "nom":"nom11", "prenom":"prenom1"},
    {"id":12, "nom":"nom12", "prenom":"prenom1"},
    {"id":13, "nom":"nom13", "prenom":"prenom1"},
    {"id":14, "nom":"nom14", "prenom":"prenom1"},
    {"id":15, "nom":"nom15", "prenom":"prenom1"},
    {"id":16, "nom":"nom16", "prenom":"prenom1"},
    {"id":17, "nom":"nom17", "prenom":"prenom1"},
    {"id":18, "nom":"nom18", "prenom":"prenom1"},
    {"id":19, "nom":"nom19", "prenom":"prenom1"},
    {"id":20, "nom":"nom20", "prenom":"prenom1"},
    {"id":21, "nom":"nom21", "prenom":"prenom1"},
    {"id":22, "nom":"nom22", "prenom":"prenom1"},
    {"id":23, "nom":"nom23", "prenom":"prenom1"}
]

```

Un tableau simple
-----------------

rien de vraiment compliqué

```html
<!-- table -->
<md-table-container>
    <table md-table>
        <thead md-head md-order="query">
            <tr md-row>
                <!-- when the user clicks this cell, the myOrder variable will get the value 'nameToLower' -->
                <th md-column md-numeric md-order-by="id">id</th>
                <!-- the variable myOrder will not be changed when this cell is clicked -->
                <th md-column md-order-by="nom">nom</th>
                <th md-column md-order-by="prenom">prénom</th>
            </tr>
        </thead>
        <tbody md-body>
            <!-- we can let ng-repeat sort the columns for us -->
            <tr  md-row  ng-repeat="data in datas | orderBy: query">
            <td md-cell>{{data.id}}</td>
            <td md-cell>{{data.nom}}</td>
            <td md-cell>{{data.prenom}}</td>
            </tr>
        </tbody>
    </table>
</md-table-container>
<!-- fin table --> 
```

Un tableau avec pagination
--------------------------

```html
<!-- table -->
<md-table-container>
    <table md-table>
        <thead md-head md-order="query.order">
            <tr md-row>
                <!-- when the user clicks this cell, the myOrder variable will get the value 'nameToLower' -->
                <th md-column md-numeric md-order-by="id">id</th>
                <!-- the variable myOrder will not be changed when this cell is clicked -->
                <th md-column md-order-by="nom">nom</th>
                <th md-column md-order-by="prenom">prénom</th>
            </tr>
        </thead>
        <tbody md-body>
            <!-- we can let ng-repeat sort the columns for us -->
            <tr  md-row  ng-repeat="data in datas | orderBy: query.order | limitTo: query.limit: (query.page - 1) * query.limit"">
            <td md-cell>{{data.id}}</td>
            <td md-cell>{{data.nom}}</td>
            <td md-cell>{{data.prenom}}</td>
            </tr>
        </tbody>
    </table>
</md-table-container>
<md-table-pagination md-limit="query.limit" md-page="query.page" md-total="{{datas.length}}" md-limit-options=query.limitoptions></md-table-pagination>
<!-- fin table --> 


```

Un tableau avec sélection simple
--------------------------------

```html
<!-- table -->
<md-table-container>
    <table md-table md-row-select ng-model="selected">
        <thead md-head md-order="query.order">
            <tr md-row>
                <!-- when the user clicks this cell, the myOrder variable will get the value 'nameToLower' -->
                <th md-column md-numeric md-order-by="id">id</th>
                <!-- the variable myOrder will not be changed when this cell is clicked -->
                <th md-column md-order-by="nom">nom</th>
                <th md-column md-order-by="prenom">prénom</th>
            </tr>
        </thead>
        <tbody md-body>
            <!-- we can let ng-repeat sort the columns for us -->
            <tr  md-row md-on-select="onSelect" md-select="data" md-select-id="id" md-auto-select ng-repeat="data in datas | orderBy: query.order | limitTo: query.limit: (query.page - 1) * query.limit">
            <td md-cell>{{data.id}}</td>
            <td md-cell>{{data.nom}}</td>
            <td md-cell>{{data.prenom}}</td>
            </tr>
        </tbody>
    </table>
</md-table-container>
<md-table-pagination md-limit="query.limit" md-page="query.page" md-total="{{datas.length}}" md-limit-options=query.limitoptions></md-table-pagination>
<!-- fin table --> 
...
<!-- javascript -->
$scope.getInit = function() {
    $scope.selected = [];
    $scope.query = {
    order: 'name',
    limit: 5,
    limitoptions: [5, 10, 15],
    page: 1
    };
        $http
        .get("./conf/data.json")
        .success(function(data, status, headers, config) {
            $scope.datas =data;                            
        })
        .error(function(data, status, headers, config) {
            alert('error');
        });
}; 
```

Un tableau avec sélection multiple
----------------------------------

```html                  
<!-- table -->
<md-table-container>
    <table  md-table md-row-select multiple ng-model="selected">
    <thead md-head md-order="query.order">
        <tr md-row>
            <!-- when the user clicks this cell, the myOrder variable will get the value 'nameToLower' -->
            <th md-column md-numeric md-order-by="id">id</th>
            <!-- the variable myOrder will not be changed when this cell is clicked -->
            <th md-column md-order-by="nom">nom</th>
            <th md-column md-order-by="prenom">prénom</th>
        </tr>
    </thead>
    <tbody md-body>
        <!-- we can let ng-repeat sort the columns for us -->
        <tr  md-row md-on-select="onSelect" md-select="data" md-select-id="id" md-auto-select ng-repeat="data in datas | orderBy: query.order | limitTo: query.limit: (query.page - 1) * query.limit">
        <td md-cell>{{data.id}}</td>
        <td md-cell>{{data.nom}}</td>
        <td md-cell>{{data.prenom}}</td>
        </tr>
    </tbody>
    </table>
</md-table-container>
<md-table-pagination md-limit="query.limit" md-page="query.page" md-total="{{datas.length}}" md-limit-options=query.limitoptions></md-table-pagination>
<!-- fin table --> 
...
<!-- javascript -->
$scope.getInit = function() {
    $scope.selected = [];
    $scope.query = {
        order: 'name',
        limit: 5,
        limitoptions: [5, 10, 15],
        page: 1
    };
        $http
        .get("./conf/data.json")
        .success(function(data, status, headers, config) {
            $scope.datas =data;                            
        })
        .error(function(data, status, headers, config) {
            alert('error');
        });
}; 

```

Un tableau avec une barre de recherche
--------------------------------------

```html  
<!-- table -->
<md-toolbar class="md-table-toolbar md-default" ng-show="!filter.show">
    <div class="md-toolbar-tools">
    <h2 class="md-title">Test</h2>
    <div flex></div>
    <md-button class="md-icon-button" ng-click="filter.show = true">
        <fa name="search fa-lg" aria-label="search"></fa>
    </md-button>
    </div>
</md-toolbar>
<md-toolbar class="md-table-toolbar md-default"  ng-show="filter.show">
    <md-input-container class="md-icon-float md-icon-right md-block">
    <label>Search</label>
    <input ng-model="query.filter" focus-on="filter.show">
    <md-icon style="display:inline-block;" ng-click="query.filter=''; filter.show = false"><fa name="close fa-lg" aria-label="close"></fa></md-icon>
    </md-input-container>
</md-toolbar>
<md-table-container>
    <table  md-table>
    <thead md-head md-order="query.order">
        <tr md-row>
            <!-- when the user clicks this cell, the myOrder variable will get the value 'nameToLower' -->
            <th md-column md-numeric md-order-by="id">id</th>
            <!-- the variable myOrder will not be changed when this cell is clicked -->
            <th md-column md-order-by="nom">nom</th>
            <th md-column md-order-by="prenom">prénom</th>
        </tr>
    </thead>
    <tbody md-body>
        <!-- we can let ng-repeat sort the columns for us -->
        <tr  md-row ng-repeat="data in datas | orderBy: query.order | filter: query.filter | limitTo: query.limit: (query.page - 1) * query.limit">
        <td md-cell>{{data.id}}</td>
        <td md-cell>{{data.nom}}</td>
        <td md-cell>{{data.prenom}}</td>
        </tr>
    </tbody>
    </table>
</md-table-container>
<md-table-pagination md-limit="query.limit" md-page="query.page" md-total="{{(datas | filter: query.filter).length}}" md-limit-options=query.limitoptions></md-table-pagination>
<!-- fin table --> 
...
<!-- javascript -->
                $scope.getInit = function() {
$scope.selected = [];
$scope.query = {
    order: 'name',
    limit: 5,
    limitoptions: [5, 10, 15],
    page: 1
};
$scope.filter = {
    show: false,                    
    options: {
        debounce: 500
    }
};
    $http
    .get("./conf/data.json")
    .success(function(data, status, headers, config) {
        $scope.datas =data;                            
    })
    .error(function(data, status, headers, config) {
        alert('error');
    });
};     
$scope.onSelect = function () {
    for (var i = $scope.selected.length; i > 0; i--) {
        alert($scope.selected[i - 1].nom);
    };
}
// init data
$scope.getInit();
});
app.directive('focusOn', function($timeout) {
    return {
        restrict : 'A',
        link : function($scope, $element, $attr) {
            $scope.$watch($attr.focusOn, function(_focusVal) {
                $timeout(function() {
                    if (_focusVal) {
                        $element.focus();
                    } else {
                        $element.blur();
                    }
                });
            });
        }
    }
});

```

Un tableau avec un export csv
-----------------------------

Pour cela nous utilisons le module ng-csv

```html  
<!-- ng-csv -->
<script type="text/javascript" src="bower_components/angular-sanitize/angular-sanitize.min.js"></script>
<script type="text/javascript" src="bower_components/ng-csv/build/ng-csv.min.js"></script>
...
<!-- table -->
<md-toolbar class="md-table-toolbar md-default" ng-show="!filter.show">
    <div class="md-toolbar-tools">
    <h2 class="md-title">Test</h2>
    <div flex></div>
    <md-button class="md-icon-button" ng-click="filter.show = true">
        <fa name="search fa-lg" aria-label="search"></fa>
    </md-button>
    <md-button class="md-icon-button" ng-csv="datas" filename="test.csv" csv-header="['id', 'nom', 'prenom']" field-separator=";">
        <fa name="download fa-lg" aria-label="download "></fa>
    </md-button>
    </div>
</md-toolbar>
<md-toolbar class="md-table-toolbar md-default"  ng-show="filter.show">
    <md-input-container class="md-icon-float md-icon-right md-block">
    <label>Search</label>
    <input ng-model="query.filter" focus-on="filter.show">
    <md-icon style="display:inline-block;" ng-click="query.filter=''; filter.show = false"><fa name="close fa-lg" aria-label="close"></fa></md-icon>
    </md-input-container>
</md-toolbar>
<md-table-container>
    <table  md-table>
    <thead md-head md-order="query.order">
        <tr md-row>
            <th md-column md-numeric md-order-by="id">id</th>
            <th md-column md-order-by="nom">nom</th>
            <th md-column md-order-by="prenom">prénom</th>
        </tr>
    </thead>
    <tbody md-body>
        <!-- we can let ng-repeat sort the columns for us -->
        <tr  md-row ng-repeat="data in datas | orderBy: query.order | filter: query.filter | limitTo: query.limit: (query.page - 1) * query.limit">
        <td md-cell>{{data.id}}</td>
        <td md-cell>{{data.nom}}</td>
        <td md-cell>{{data.prenom}}</td>
        </tr>
    </tbody>
    </table>
</md-table-container>
<md-table-pagination md-limit="query.limit" md-page="query.page" md-total="{{(datas | filter: query.filter).length}}" md-limit-options=query.limitoptions></md-table-pagination>
<!-- fin table --> 
<!-- fin table --> 
...
<!-- javascript -->
var app = angular.module('MyAppli', ['picardy.fontawesome','ngMaterial', 'md.data.table', 'ngSanitize', 'ngCsv']);

```

Un tableau avec colonnes visibles ou pas
----------------------------------------

L'idée est de pouvoir sélectionner les colonnes à afficher

```html    
<!-- table -->
<md-toolbar class="md-table-toolbar md-default" >
    <div class="md-toolbar-tools">
    <h2 class="md-title">Test</h2>
    <div flex></div>
    <md-menu>
        <md-button class="md-icon-button" ng-click="$mdOpenMenu($event)">
            <fa name="ellipsis-v fa-lg" aria-label="columns"></fa>
        </md-button>
            <md-menu-content width="4">
            <md-list ng-cloak>
            <md-list-item>
                <p> id </p>
                <md-checkbox class="md-secondary" ng-model="columns.id.show"></md-checkbox>
            </md-list-item>
            <md-list-item>
                <p> nom </p>
                <md-checkbox class="md-secondary" ng-model="columns.nom.show"></md-checkbox>
            </md-list-item>
            <md-list-item>
                <p> prenom </p>
                <md-checkbox class="md-secondary" ng-model="columns.prenom.show"></md-checkbox>
            </md-list-item>
            </md-list>
            </md-menu-content>
    </md-menu>
    </div>
</md-toolbar>
<md-table-container>
    <table  md-table>
    <thead md-head md-order="query.order">
        <tr md-row>
            <th md-column md-numeric md-order-by="id" ng-show="columns.id.show">id</th>
            <th md-column md-order-by="nom" ng-show="columns.nom.show">nom</th>
            <th md-column md-order-by="prenom" ng-show="columns.prenom.show">prénom</th>
        </tr>
    </thead>
    <tbody md-body>
        <!-- we can let ng-repeat sort the columns for us -->
        <tr  md-row ng-repeat="data in datas | orderBy: query.order">
        <td md-cell ng-show="columns.id.show">{{data.id}}</td>
        <td md-cell ng-show="columns.nom.show">{{data.nom}}</td>
        <td md-cell ng-show="columns.prenom.show">{{data.prenom}}</td>
        </tr>
    </tbody>
    </table>
</md-table-container>
<!-- fin table --> 
...
<!-- javascript -->
$scope.columns={
    "id" : {"show": true},
    "nom" : {"show": true},
    "prenom" : {"show": false},
};

```

Un tableau avec toutes les fonctionnalitées et dynamique
--------------------------------------------------------

Cette fois ci le code est générique et prend en charge tous les tableaux

```html
<!DOCTYPE html>
<html lang="fr">
    <head>
        <title>All functions</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, user-scalable=no, initial-scale=1.0, maximum-scale=1.0, minimal-ui" />
        <!-- Angular -->
        <script src="./bower_components/angular/angular.min.js"></script>
        <!-- Angular Material -->
        <script src="./bower_components/angular-animate/angular-animate.min.js"></script>
        <script src="./bower_components/angular-aria/angular-aria.min.js"></script>
        <script src="./bower_components/angular-material/angular-material.min.js"></script>
        <link rel="stylesheet" href="./bower_components/angular-material/angular-material.min.css" />
        <!-- Angular FontAwesome -->
        <link rel="stylesheet" href="./bower_components/font-awesome/css/font-awesome.min.css">
        <script src="./bower_components/angular-fontawesome/dist/angular-fontawesome.min.js"></script>
        <!-- md-table -->
        <link href="bower_components/angular-material-data-table/dist/md-data-table.min.css" rel="stylesheet" type="text/css"/>
        <script type="text/javascript" src="bower_components/angular-material-data-table/dist/md-data-table.min.js"></script>
        <!-- ng-csv -->
        <script type="text/javascript" src="bower_components/angular-sanitize/angular-sanitize.min.js"></script>
        <script type="text/javascript" src="bower_components/ng-csv/build/ng-csv.min.js"></script>
        <!-- mycss -->
        <link rel="stylesheet" href="./css/main.css">
        
    </head>
    <body ng-app="MyAppli">
        <div ng-controller="AppController" layout="column" layout-fill>
        <div ui-view="content" layout-fill layout="column" layout-align="center none">
                    <!-- table -->
                    <md-toolbar class="md-table-toolbar md-default"  ng-show="!filter.show">
                    <div class="md-toolbar-tools">
                        <h2 class="md-title">Test</h2>
                        <div flex></div>
                        <md-button class="md-icon-button" ng-click="filter.show = true">
                        <fa name="search fa-lg" aria-label="search"></fa>
                        </md-button>
                        <md-button class="md-icon-button" ng-csv="getDatasNgCsv" filename="test.csv"  field-separator=";" >
                        <fa name="download fa-lg" aria-label="download "></fa>
                        </md-button>
                        <md-menu>
                            <md-button class="md-icon-button" ng-click="$mdOpenMenu($event)">
                            <fa name="ellipsis-v fa-lg" aria-label="columns"></fa>
                            </md-button>
                            <md-menu-content width="4">
                                <md-list ng-cloak>
                                <md-list-item ng-repeat="column in columns">
                                <p> {{ column.title}} </p>
                                <md-checkbox class="md-secondary" ng-model="column.show"></md-checkbox>
                                </md-list-item>
                                </md-list>
                            </md-menu-content>
                        </md-menu>
                    </div>
                    </md-toolbar>
                    <md-toolbar class="md-table-toolbar md-default"  ng-show="filter.show">
                    <md-input-container class="md-icon-float md-icon-right md-block">
                        <label>Search</label>
                        <input ng-model="query.filter" focus-on="filter.show">
                        <md-icon style="display:inline-block;" ng-click="query.filter=''; filter.show = false"><fa name="close fa-lg" aria-label="close"></fa></md-icon>
                    </md-input-container>
                    </md-toolbar>
                    <md-table-container>
                    <table  md-table md-row-select multiple ng-model="selected">
                        <thead md-head md-order="query.order">
                        <tr md-row>
                            <th md-column ng-repeat="column in columns" ng-show="column.show">{{ column.title}}</th>
                        </tr>
                        </thead>
                        <tbody md-body>
                        <!-- we can let ng-repeat sort the columns for us -->
                        <tr  md-row md-on-select="onSelect" md-select="data" md-select-id="id" md-auto-select ng-repeat="data in datas | orderBy: query.order | filter: query.filter | limitTo: query.limit: (query.page - 1) * query.limit">
                            <td md-cell ng-repeat="column in columns" ng-show="column.show">{{ data[column.id]}}</td>
                        </tr>
                        </tbody>
                    </table>
                    </md-table-container>
                    <md-table-pagination md-limit="query.limit" md-page="query.page" md-total="{{(datas | filter: query.filter).length}}" md-limit-options=query.limitoptions></md-table-pagination>
        </div>
        </div>
        <script>
            var app = angular.module('MyAppli', ['picardy.fontawesome','ngMaterial', 'md.data.table', 'ngSanitize', 'ngCsv']);
            app.controller('AppController', function($scope, $http) {
            
                $scope.getInit = function() {
                $scope.selected = [];
                $scope.query = {
                    order: 'name',
                    limit: 5,
                    limitoptions: [5, 10, 15],
                    page: 1
                };
                $scope.filter = {
                    show: false,                    
                    options: {
                    debounce: 500
                    }
                };  
                    $http
                        .get("./conf/data.json")
                        .success(function(data, status, headers, config) {
                            $scope.datas =data;   
                            $scope.columns=[];    
                            for (key in data[0]) {
                                $scope.columns.push({"id" : key, "title": key, "show": true});
                            };                 
                        })
                        .error(function(data, status, headers, config) {
                            alert('error');
                        });
                };      
                $scope.getDatasNgCsv = function () {
                var tmp = [];
                headercsv={};     
                for (key in $scope.datas[0]) {
                    headercsv[key] = key;
                };
                tmp.push(headercsv);
                for (var i = 0; i < $scope.datas.length; i++) {
                    tmp.push($scope.datas[i]);
                };
                return tmp;
                };   
                $scope.onSelect = function () {
                for (var i = $scope.selected.length; i > 0; i--) {
                    alert($scope.selected[i - 1].nom);
                };
                };
                // init data
                $scope.getInit();
            });
            app.directive('focusOn', function($timeout) {
                return {
                    restrict : 'A',
                    link : function($scope, $element, $attr) {
                        $scope.$watch($attr.focusOn, function(_focusVal) {
                            $timeout(function() {
                                if (_focusVal) {
                                    $element.focus();
                                } else {
                                    $element.blur();
                                }
                            });
                        });
                    }
                }
            });
        </script>
    </body>
</html>

```

Conclusion
----------

md-table (https://github.com/daniel-nagy/md-data-table) est un super module qui permet encore beaucoup de chose comme l'édition,
module à intégrer dès que possible sur les projets.